<?php
/**
 * @package 
 * @version 1.1.2 
 * @author Jason Lau
 * @link http://jasonlau.biz
 * @copyright 2011
 * @license GNU/GPL 3+
 * @uses WordPress
 
Plugin Name: Visitor Maps Extended Referer Field
Plugin URI: http://jasonlau.biz
Description: This WordPress plugin extends and depends on the <a href="http://www.642weather.com/weather/scripts-wordpress-visitor-maps.php" target="_blank">Visitor Maps and Who's Online</a> plugin by <a title="Visit author homepage" href="http://www.642weather.com/weather/scripts.php" target="_blank">Mike Challis</a>. This plugin alters the <em>Referer</em> column for the Admin's <em>Who's Been Online</em> grid, so it displays the referring host name followed by the search string, if there is one. It also prevents long URLs from expanding the grid too wide for viewing. Additionally, you can ban and unban visitor IP addresses using a toggle button. Banned IP addresses are added to .htaccess, which is automatically backed-up before any changes are made. There are no settings required.
Author: Jason Lau
Version: 1.1.2
Author URI: http://jasonlau.biz
*/

if (realpath(__FILE__) == realpath($_SERVER['SCRIPT_FILENAME'])) {
    exit('Please don\'t access this file directly.');
}


function visitor_maps_extended_ref(){
    wp_enqueue_script('jquery');
    wp_register_script('visitor_maps_extended_referer', plugins_url('/visitor-maps-extended-referer-field/visitor-maps-extended-referer.js', dirname(__FILE__)));
    wp_enqueue_script('visitor_maps_extended_referer');
    vmerf_init();
}

function vmerf_init(){
    $vmerf_output = "<script type=\"text/javascript\">
	var vmerf_message = '', vmerf_ip = '', vmerf_admin_ip = '" . $_SERVER["REMOTE_ADDR"] . "';\n";
    
    if($_POST['vmerf_mode']){
      switch ($_POST['vmerf_mode']){        
        case "ban referer":
        $vmerf_output .= vmerf_ban_referer();
        break;
        
        case "unban referer":
        $vmerf_output .= vmerf_unban_referer();
        break;
        
        case "purge referers":
        $vmerf_output .= vmerf_purge_referers();
        break;
        
        case "ban":
        $vmerf_output .= vmerf_ban_ip();
        break;
        
        case "unban":
        $vmerf_output .= vmerf_unban_ip();
        break;
        
        case "purge":
        $vmerf_output .= vmerf_purge_ips();
        break;
      }  
    }
    
    $banned_ips = get_option("vmerf_banned_ips");   
    $banned_referers = get_option("vmerf_banned_referers");
    $vmerf_output .= "var vmerf_banned_ips = '" . @implode(', ', $banned_ips) . "';
        vmerf_banned_ips = vmerf_banned_ips.split(', ').sort();\n";
    $vmerf_output .= "var vmerf_banned_referers = '" . @implode(', ', $banned_referers) . "';
        vmerf_banned_referers = vmerf_banned_referers.split(', ').sort();\n";
    $vmerf_output .= "var vmerf_admin_host = '" . $_SERVER["HTTP_HOST"] . "';\n";
    $vmerf_output .= "</script>\n";
    
    echo $vmerf_output;
}

function vmerf_new_list($referer){
    $new_list = "# BEGIN Referers
<IfModule mod_rewrite.c>
# Uncomment 'Options +FollowSymlinks' if your server returns a '500 Internal Server' error.
# This means your server is not configured with FollowSymLinks in the '' section of the 'httpd.conf'.
# Contact your system administrator for advice with this issue.
# Options +FollowSymlinks
# Cond start
RewriteCond %{HTTP_REFERER} " . str_replace(".", "\.", $referer) . " [NC]
# Cond end
RewriteRule .* - [F]
</IfModule>
# END Referers";
    return $new_list;
}

function vmerf_ban_referer(){
    $vmerf_referer = eregi_replace("http://", "", eregi_replace("www.", "", trim($_POST['vmerf_referer'])));
    $banned_referers = get_option("vmerf_banned_referers");
    $vmerf_output = "";
    if(in_array($vmerf_referer, $banned_referers)){
        $vmerf_output .= "vmerf_message += '" . $vmerf_referer . " is already banned!';\n";  
    } else {
        array_push($banned_referers, $vmerf_referer);
        update_option("vmerf_banned_referers", $banned_referers);
        vmerf_rebuild_htaccess();
        $vmerf_output .= "vmerf_message += '" . $vmerf_referer . " successfully banned!';\n";
    }
    $vmerf_output .= "var vmerf_mode = 'ban referer';\n";
    return $vmerf_output;  
}

function vmerf_unban_referer(){
    $vmerf_referer = trim($_POST['vmerf_referer']);
    $banned_referers = get_option("vmerf_banned_referers");
    $vmerf_output = "";
    $new_list = array();
    foreach($banned_referers as $referer){
        if($referer != $vmerf_referer) array_push($new_list, $referer);
    }
    update_option("vmerf_banned_referers", $new_list);
    vmerf_rebuild_htaccess();
    $vmerf_output .= "vmerf_message += '" . $vmerf_referer . " successfully unbanned!';\n";
    $vmerf_output .= "var vmerf_mode = 'unban referer';\n";
    return $vmerf_output;  
}

function vmerf_purge_referers(){
    update_option("vmerf_banned_referers", array());
    vmerf_rebuild_htaccess();
    $vmerf_output .= "vmerf_message += 'All referers successfully unbanned!';\n";
    $vmerf_output .= "var vmerf_mode = 'purge referers';\n";
    return $vmerf_output;  
}

function vmerf_ban_ip(){
    $banned_ips = get_option("vmerf_banned_ips");
    $vmerf_ip = trim($_POST['vmerf_ip']);
    $vmerf_output = "";
    if(in_array($vmerf_ip, $banned_ips)){
        $vmerf_output .= "vmerf_message += '" . $vmerf_ip . " is already banned!';\n";  
    } else {
        array_push($banned_ips, $vmerf_ip);
        update_option("vmerf_banned_ips", $banned_ips); 
        vmerf_rebuild_htaccess();             
        $vmerf_output .= "vmerf_message += '" . $vmerf_ip . " successfully banned!';\n";
    }
    $vmerf_output .= "vmerf_ip += '" . $vmerf_ip . "';\n";
    return $vmerf_output;  
}

function vmerf_unban_ip(){
    $banned_ips = get_option("vmerf_banned_ips");
    $vmerf_ip = trim($_POST['vmerf_ip']);
    $vmerf_output = "";
    $vmerf_output .= "var vmerf_mode = 'unban';\n";
    $new_list = array();
    foreach($banned_ips as $ip){
        if(trim($ip) != $vmerf_ip){
            array_push($new_list, trim($ip));
        } 
    }
    update_option("vmerf_banned_ips", $new_list);
    vmerf_rebuild_htaccess();
    $vmerf_output .= "vmerf_message += '" . $vmerf_ip . " successfully unbanned!';\n";
    $vmerf_output .= "vmerf_ip += '" . $vmerf_ip . "';\n";
    return $vmerf_output;  
}

function vmerf_purge_ips(){
    update_option("vmerf_banned_ips", array());
    vmerf_rebuild_htaccess();
    $vmerf_output .= "vmerf_message += 'All IP addresses successfully unbanned!';\n";
    return $vmerf_output;  
}

function vmerf_read(){
    $fp = fopen(ABSPATH . ".htaccess", 'rt');
    $content = "";
    while(!feof($fp)){
        $content .= fgets($fp, 4096);
    }
    fclose($fp);
    return $content; 
}

function vmerf_write($content){
    $fp = fopen(ABSPATH . ".htaccess", "w+");
    fwrite($fp, trim($content));
    fclose($fp);
}

function vmerf_del_line_in_file($filename, $text_to_delete){
  $file_array = array();	
  $file = fopen($filename, 'rt');
  if($file){
    while(!feof($file)){
      $val = fgets($file);
      if(is_string($val))
        array_push($file_array, $val);
    }	
    fclose($file);
  }
  for($i = 0; $i < count($file_array); $i++){
    if(eregi($text_to_delete, $file_array[$i])){
      if(($file_array[$i] == $text_to_delete . "\n") || ($file_array[$i] == $text_to_delete)) $file_array[$i] = '';
    }
  }
  $content = trim(implode("", $file_array));
  if($text_to_delete == "all referers"){
    $all_referers = substr($content, stripos($content,"# BEGIN Referers"),stripos($content,"# END Referers"));
    $content = trim(str_replace($all_referers, "", $content));
    update_option("vmerf_referers", array());
  }
  
  $file_write = fopen($filename, 'wt');	
  if($file_write){
    fwrite($file_write, $content);
    fclose($file_write);
  }
}


function vmerf_create_random($number_of_digits = 1, $type = 3){
    // type: 1 - numeric, 2 - letters, 3 - mixed, 4 - all ascii chars.
    for ($x = 0; $x < $number_of_digits; $x++) {
        while (substr($num, strlen($num) - 1, strlen($num)) == $r) {
            switch ($type) {
                case "1":
                    $r = rand(0, 9);
                    break;

                case "2":
                    $n = rand(0, 999);
                    if ($n % 2) {
                        $r = chr(rand(0, 25) + 65);
                    } else {
                        $r = strtolower(chr(rand(0, 25) + 65));
                    }
                    break;

                case "3":
                    if (is_numeric(substr($num, strlen($num) - 1, strlen($num)))) {
                        $n = rand(0, 999);
                        if ($n % 2) {
                            $r = chr(rand(0, 25) + 65);
                        } else {
                            $r = strtolower(chr(rand(0, 25) + 65));
                        }
                    } else {
                        $r = rand(0, 9);
                    }
                    break;
                    
                    case "4":
                    if (is_numeric(substr($num, strlen($num) - 1, strlen($num)))) {
                        $n = rand(0, 999);
                        if ($n % 2) {
                            $r = chr(rand(33, 231));
                        } else {
                            $r = strtolower(chr(rand(33, 231)));
                        }
                    } else {
                        $r = rand(33, 231);
                    }                   
                    break;
            }
        }
        $num .= $r;
    }
    return $num;
}

function vmerf_install(){
    global $wp_version;
    $htbackup = ABSPATH . ".htaccess.backup." . vmerf_create_random(6);
    update_option("vmerf_htbackup", $htbackup);
    if(!@copy(ABSPATH . ".htaccess", $htbackup)){
        wp_die("Visitor Maps Extended could not backup " . ABSPATH . ".htaccess. The installation was terminated.");
    } else {
        $vmerf_banned_ips = get_option("vmerf_banned_ips");
        if($vmerf_banned_ips != "" && !is_array($vmerf_banned_ips)){
          $vmerf_banned_ips_array = @explode(", ", $vmerf_banned_ips);
          if($vmerf_banned_ips_array[0] != ""){
            foreach($vmerf_banned_ips_array as $ip){
                vmerf_del_line_in_file(ABSPATH . ".htaccess", "deny from " . trim($ip));
            }
            update_option("vmerf_banned_ips", $vmerf_banned_ips_array);
          } else {
            update_option("vmerf_banned_ips", array());
          }  
        } else {
            
        }
        update_option("vmerf_banned_ips", array());
        if(!get_option("vmerf_banned_referers")){
            add_option("vmerf_banned_referers", array());
        }      
        update_option("vmerf_wp_version", $wp_version);
        vmerf_rebuild_htaccess(); 
    }         
}

function vmerf_deactivate(){
    // Preserve data
    /*copy(ABSPATH . ".htaccess", get_option("vmerf_htbackup")); 
    update_option("vmerf_banned_ips", array());
    update_option("vmerf_banned_referers", array());
    vmerf_rebuild_htaccess();  
    delete_option("vmerf_wp_version");
    delete_option("vmerf_banned_ips");
    delete_option("vmerf_banned_referers");
    delete_option("vmerf_htbackup");*/    
}

function vmerf_rebuild_htaccess(){
    copy(ABSPATH . ".htaccess", get_option("vmerf_htbackup"));
    $htcontent = vmerf_read();
    if(@eregi("Visitor Maps Extended", $htcontent)){
        $part1 = explode("# BEGIN Visitor Maps Extended", $htcontent);
        $part2 = explode("# END Visitor Maps Extended", $htcontent);        
        $htpart1 = trim($part1[0]);
        $htpart2 = trim($part2[1]);        
    } else {
      $htpart1 = trim($htcontent);
      $htpart2 = "";
    }
    $banned_referers = get_option("vmerf_banned_referers");
    $banned_ips = get_option("vmerf_banned_ips");
    $num_referers = count($banned_referers);
    $new_htcontent = "\n# BEGIN Visitor Maps Extended";
    if($banned_referers[0] != ""){
        $new_htcontent .= "\n# BEGIN Referers
<IfModule mod_rewrite.c>
# Uncomment 'Options +FollowSymlinks' if your server returns a '500 Internal Server' error.
# This means your server is not configured with FollowSymLinks in the '' section of the 'httpd.conf'.
# Contact your system administrator for advice with this issue.
# Options +FollowSymlinks
# Cond start ";
    
    foreach($banned_referers as $referer){
        if(trim($referer) != ""){
            if($num_referers == "1"){
                $new_htcontent .= "\nRewriteCond %{HTTP_REFERER} " . str_replace(".", "\.", $referer) . " [NC]";
            } else {
                $new_htcontent .= "\nRewriteCond %{HTTP_REFERER} " . str_replace(".", "\.", $referer) . " [NC,OR]";
            }
        }
    }
    if($num_referers != "1"){
        $new_htcontent = trim($new_htcontent, "[NC,OR]");
    }
    $new_htcontent .= "\n# Cond end
RewriteRule .* - [F]
</IfModule>
# END Referers";
    }
    if($banned_ips[0] != ""){
      $new_htcontent .= "\n# BEGIN banned ips
<Limit GET POST>
order allow,deny";
    foreach($banned_ips as $ip){
        if(trim($ip) != ""){
            $new_htcontent .= "\ndeny from " . $ip;  
        }          
    }
    $new_htcontent .= "\nallow from all
</Limit>
# END banned ips";  
    }    
    $new_htcontent .= "\n# END Visitor Maps Extended\n";
    if(count($banned_referers) == 0 && count($banned_ips) == 0){
        $new_htcontent = "";
    }
    vmerf_write($htpart1 . $new_htcontent . $htpart2);
}

if(is_dir(plugin_dir_path('visitor-maps')) && @eregi("whos-been-online", $_REQUEST['page'])){
    add_action('admin_enqueue_scripts', 'visitor_maps_extended_ref'); 
}

register_activation_hook(__FILE__, 'vmerf_install');
register_deactivation_hook(__FILE__, 'vmerf_deactivate');

?>